########################################################################
## Heterogeneous treatment effects > Djibouti dataset preparation 
## author : Morgane Hoffmann and Elia Perennes
# Last version : 02/07/2022 (EG adaptation for endline 01/09/2022)
#######################################################################


#### Install packages 

vec.pac= c(
  
  # Opening files   
  'haven','sjlabelled','stringr'
  
) 

package.check <- lapply(
  vec.pac,
  FUN = function(x) {
    if (!require(x, character.only = TRUE)) {
      install.packages(x, dependencies = TRUE)
      library(x, character.only = TRUE)
    }
  }
)

# Load functions from external files 
# Import helper functions 
source("codes/0_helper_functions.R")


#########################################################
# 1. DATA CLEANING
#########################################################

#-------------------------------------------------------
# 1.1 Get data
#-------------------------------------------------------

# Open raw data (also put in data here but you c)
file_name <- "midline_analysis.dta"

data <- data.frame(read_dta(paste("data/",file_name,sep="/")))

#-------------------------------------------------------
# 1.2 Set parameters
#-------------------------------------------------------

# specify cluster, fixed effect and partition  
cluster      <- "0"       # if no cluster use cluster <- "0"
fixed_effect <- "0"         # if no fixed_effect use fixed_effect <- "0"
partition <- "0"  # If you need to construct stratified samples for auxiliary and main

# Specify treatment variable 
treatment <- 'treatment'

# Specify id variable, if non it will be generated
id <- '0'

#-------------------------------------------------------
# 1.3 Clean controls
#-------------------------------------------------------

# We save control variables (see codebook)
control_variables <- c('bs_hh_enfant2','bs_enfant35','bs_enfant615','bs_adults','bs_headmale','bs_headage','bs_headnoeduc',
                       'bs_headliteracy2','bs_benefage','bs_benefsomali','bs_benefafar','bs_benefnoeduc','bs_benefliteracy2',
                       'bs_benefpregnant','bs_benefmarried','bs_childinschool615','bs_tele','bs_frigo','bs_cuisi_charb','bs_cuisi_gaz',
                       'bs_proprio_terre','bs_proprio_maison','bs_men_shemployed','bs_men_shunemployed','bs_men_shsalaried','bs_men_shdayworker',
                       'bs_men_shself','bs_wo_shemployed','bs_wo_shunemployed','bs_wo_shsalaried','bs_wo_shdayworker','bs_wo_shself','bs_income_pension',
                       'bs_income_govngo','bs_income_transfert','bs_transfer_extrahh','bs_foodinsecurity','bs_lexppc_total','bs_lexppc_food','bs_wo_employed',
                       'bs_wo_inactive','bs_wo_daywork','bs_wo_self','bs_wo_salaried','bs_index_womendecision_fa','bs_index_mobility_fa','bs_w_low_selfesteem',
                       'bs_quint5_exppc_food','bs_w_selfesteem')
controls <- data[,control_variables]

# Set labels for control variables 
controls <- controls %>%
  var_labels(bs_benefage = "Age of Woman Beneficiary", 
             bs_men_shemployed =  "Share of adult men who have worked in last 7 days",
             bs_wo_employed = "Beneficiary women has worked in last 7 days",
             bs_index_womendecision_fa = "Index of Woman's decision making - factor analysis", 
             bs_index_mobility_fa = "Index of Woman's mobility - factor analysis",
             bs_w_selfesteem = "Indicator for self-esteem, beneficiary woman")


# Choose the variables you want to include in the CLAN
clan_variables <- c('bs_benefage', 'bs_men_shemployed', 'bs_wo_employed', 'bs_index_womendecision_fa', 'bs_index_mobility_fa', 'bs_w_selfesteem')

control_variables_renamed <- c()
for(element in control_variables){
  if (element %in% clan_variables){
    element <- paste(element, '_c', sep='')
  }
  control_variables_renamed <- c(control_variables_renamed, element)
}

# Harmonisation :  all control variables will start with "c_" 
control_variables <- control_variables_renamed %>% str_replace("bs_", "c_")
names(controls) <- control_variables

# Number of control variables 
cat('\nNumber of control variables : ',length(controls),'\n')

# Impute medians for all missing observationsin numeric variables (taken from old script)
for(col in control_variables) {                                   # Replace NA in all columns
  controls[ , col][is.na(controls[ , col])] <- median(controls[ , col], na.rm = TRUE)
}

# Check variable category
covariate_type <- identify_categ_bin(controls)
cont_var <- covariate_type[[1]]
categ_var <-covariate_type[[2]]
bin_var <-covariate_type[[3]]

cat('We have ',length(categ_var), ' categorical variables \n')
cat('We have ',length(cont_var), ' continuous variables \n')
cat('We have ',length(bin_var), ' binary variables \n')

controls <- categ_to_factors(controls,categ_var) # bin_var
controls <- categ_to_factors(controls,bin_var) 


#-------------------------------------------------------
# 1.4 Clean outcomes
#-------------------------------------------------------

outcome_variables <- c('w123w_mean_employed',"w123m_mean_employed", "w123w_mean_remun", "lw123_mean_income_total", "lexppc_total", "lexppc_food")

outcomes <- data[,outcome_variables]

# Set labels for control variables 
outcomes <- outcomes %>%
  var_labels(w123w_mean_employed = "Woman worked", 
             w123m_mean_employed =  "Husband worked",
             w123w_mean_remun = "Women's remuneration",
             lw123_mean_income_total = "Total income (ln)", 
             lexppc_total = "Per capita total expenditures (ln)",
             lexppc_food = "Per capita food expenditures (ln)")

# Harmonisation :  all control variables will start with "y_" 
outcome_variables <- paste('y', outcome_variables, sep='_')
colnames(outcomes) <- outcome_variables

# We are going to winsorize continuous outcomes 
#hist(outcomes$y_w123w_mean_remun, breaks=100)
# A lot of zeros with some extreme values 
# We wnsorize at 0.99
summary(outcomes['y_w123w_mean_remun'])
q9 <- quantile(outcomes['y_w123w_mean_remun'], c(0.95), na.rm=TRUE)
outcomes$y_w123w_mean_remun[outcomes$y_w123w_mean_remun > q9] <- NA

colnames(outcomes)


#-------------------------------------------------------
# 1.5 Other 
#-------------------------------------------------------


data_clean <- cbind(outcomes, controls)

if(cluster!="0"){
  clust <- data.frame(data[, cluster])
  colnames(clust) <- paste('clust', cluster, sep='_')
  data_clean <- cbind(data_clean, clust)
}

if(partition!="0"){
  part <- data.frame(data[, partition])
  colnames(part) <- paste('part', partition, sep='_')
  data_clean <- cbind(data_clean, part)
  
}

if(fixed_effect!="0"){
  fe <- data[, fixed_effect]
  colnames(fe) <- paste('fe', fixed_effect, sep='_')
  data_clean <- cbind(data_clean, fe)
}


cat("Number of observations: ",dim(data_clean)[1])
cat("Number of variables in the dataset: ", dim(data_clean)[2])

rownames(data_clean)<- NULL

# Harmonisation : treatment variable will be called "treatment"
treat_var <- data[treatment]
colnames(treat_var) <- 'treatment'
data_clean <- cbind(data_clean, treat_var)

# Set id 
if(id!="0"){
  id_var <- data_clean[id]
  colnames(id_var) <- 'id'
  data_clean <- cbind(data_clean, id_var)
}else{
  data_clean$id <- as.numeric(row.names(data_clean))
}

# Last check
summary(data_clean)

data <- data_clean

#-------------------------------------------------------
# 1.6 Save
#-------------------------------------------------------
data_path = "data/"
# filename_out = 'djibouti_followup.RData'
data_name = 'djibouti_midline'
save(data, file = paste0(data_path, paste0(data_name, ".RData", sep='')))

