########################################################################
## Heterogeneous treatment effects > Madagascar Mahay and Djibouti
## Version : 2
## author : Morgane Hoffmann and Elia Perennes
# Last version : 03/11/2023
#######################################################################


# -----------------------------------------------------------------------#
# 0. INITIALIZATION
# 0.1 Load packages 
# 0.2 Load custom functions
# 0.3 Open your dataset
# 0.4 Set parameters for the general procedure
# 0.5 Create saving files

# 1. DATA AND SAMPLE PREPARATION 
# 1.1 Identify variables in data
# 1.2 Create AUX/MAIN samples

# 2. MACHINE LEARNING MODELS PREPARATION
# 2.1 Machine Learning methods to be tested
# 2.2 Tuning parameters for ML models

# 3. CONSTRUCT ML PROXIES
# 3.1 Loop to construct ML proxy predictors (THIS IS LONG)

# 4. REGRESSIONS
# 4.1 Recover all saved B and S
# 4.2 Prepare data for regressions
# 4.3 Regression loop
# 4.4 Aggregate results into table and save

# -----------------------------------------------------------------------------#


gml = function(data_name,
               name_of_experiment,
               y,
               nb_groups,
               treatment_arms_num,
               control_arms_num,
               sim,
               methods,
               run_ml) {

  
  #########################################################
  # 0. INITIALIZATION
  #########################################################
  
  #-------------------------------------------------------
  # 0.1 Load packages 
  #-------------------------------------------------------
  
  # Load packages 
  source("codes/0_install_packages.R")
  
  
  #-------------------------------------------------------
  # 0.2 Load custom functions
  #-------------------------------------------------------
  
  source("codes/0_helper_functions.R")
  source("codes/2b_methods_functions.R")
  
  #-------------------------------------------------------
  # 0.3 Open your dataset
  #-------------------------------------------------------
  
  # Open data
  file_name <- paste('data/', data_name, ".RData", sep = '')
  load(file_name)
  
  cat("\n Number of observations: ", dim(data)[1])
  cat("\n Number of variables in the dataset: ", dim(data)[2])
  cat('\n Control variables are the following : \n\n' , colnames(data)[startsWith(colnames(data), 'c_')])

  treatment_arm_name = paste(
    paste(control_arms_num, collapse = ""),
    "vs",
    paste(treatment_arms_num, collapse = ""),
    sep = ''
  )
  cat('\n We are comparing :' , treatment_arm_name)
  
  
  #-------------------------------------------------------
  # 0.4 Set parameters for the general procedure
  #-------------------------------------------------------
  
  # Set GML parameters
  sim     <- sim      # number of repetitions
  K       <- 2       # number of folds
  p       <- nb_groups       # number of groups
  thres   <- 1 / p     # quantile for most/least affected group
  alpha   <- 0.05    # signifigance level
  
  #-------------------------------------------------------
  # 0.5 Create saving files
  #-------------------------------------------------------
  # We create a list of paths to save or load results (remember the structure)
  path_list <- create_files(name_of_experiment, y, treatment_arm_name)
  
  
  #########################################################
  # 1. DATA AND SAMPLE PREPARATION 
  #########################################################
  
  #-------------------------------------------------------
  # 1.1 Identify variables in data
  #-------------------------------------------------------
  
  # Get all control variables, they all have a "c_" prefix in order to recognize them
  controls <- colnames(data %>% dplyr::select(starts_with("c_")))
  
  # Drop treatments not included
  data <- subset(data, treatment %in% c(treatment_arms_num, control_arms_num))
  rownames(data) <- NULL
  
  # Attribute control, treatment
  d <- "treatment"
  data$treatment <- ifelse(data$treatment %in% treatment_arms_num, 1, 0)
  
  # Are there any cluster ? If yes cluster variable should start with "clust_"
  if (length(colnames(data %>% dplyr::select(starts_with("clust_")))) == 0) {
    cluster <- '0'
  } else{
    cluster <- colnames(data %>% dplyr::select(starts_with("clust_")))[1]
  }
  
  # Are there any partition varible ? If yes partition variable should start with "part_"
  if (length(colnames(data %>% dplyr::select(starts_with("part_")))) == 0) {
    partition <- '0'
  } else{
    partition <-
      colnames(data %>% dplyr::select(starts_with("part_")))[1]
  }
  
  # Are there any fixed effect ? If yes fe variable should start with "fe_"
  if (length(colnames(data %>% dplyr::select(starts_with("fe_")))) == 0) {
    fixed_effect <- '0'
  } else{
    fixed_effect <-
      colnames(data %>% dplyr::select(starts_with("fe_")))[1]
  }
  
  covariate_type <- identify_categ_bin(data[, controls])
  cont_var <- covariate_type[[1]]
  categ_var <- covariate_type[[2]]
  bin_var <- covariate_type[[3]]
  
  # Outcome variable
  outcome <- data[y]
  
  #We drop other potential outcome variables 
  data <- data %>% dplyr::select(-starts_with("y_"))
  data <- cbind(outcome, data)
  
  # Check for missing values 
  cat('\n Missing values in outcome variable \n ')
  n_original_data <- nrow(data)
  
  # Drop NaN in the outcome
  midline_out <- data.frame(data[!complete.cases(data[y]), ])
  data   <- data.frame(data[complete.cases(data[y]), ])
  n_complete_data <- nrow(data)
  
  ## Print 
  # cat('\nNumber of observations at the beginning: ', n_original_data)
  # cat('\nNumber of observations after deleting missing values: ',
  #     n_complete_data)
  # cat(paste(
  #   "\nNumber of missing outcomes:",
  #   as.character(n_original_data - n_complete_data),
  #   sep = " "
  # ))
  
  # Drop Nan in covariates 
  n_complete_data <- nrow(data)
  data <- data[complete.cases(data), ]
  
  # Print 
  # cat('\n \n Missing values in controls \n ')
  # cat('\nNumber of observations at the beginning: ', n_complete_data)
  # cat('\nNumber of observations after deleting missing values: ',
  #     dim(data)[1])
  # cat(paste(
  #   "\nNumber of missing data :",
  #   as.character(n_complete_data - dim(data)[1]),
  #   sep = " "
  # ))
  
  
  #-------------------------------------------------------
  # 1.2 Create AUX/MAIN samples
  #-------------------------------------------------------
  
  # For some reason we need to put the seed again
  set.seed(2022)
  if (cluster == '0') {
    auxil <-
      createMultiFolds(data[, 'id'], K, times = sim) # Create stratified samples
  } else{
    auxil <-
      createMultiFolds(data[, cluster], K, times = sim) # Create stratified samples
  }
  
  # At each simulation we save sample row index for each, we also flag the seed used !
  name_sample_seed <-
    paste(paste('aux', seed, sep = '_'), '.RData', sep = "")
  save(auxil,
       file = paste(path_list$sample_folder, name_sample_seed, sep = '/'))
       
       
       
 #########################################################
 # 2. MACHINE LEARNING MODELS PREPARATION
 #########################################################

       
  #-------------------------------------------------------
  # 2.1 Machine Learning methods to be tested
  #-------------------------------------------------------
  
  # Machine learning methods available :
  method_names <- list(
    # Two learners
    glmnet = "Elastic-Net",
    glmnet_exp = "Elastic-Net with Expansion",
    gbm = "Boosting",
    rf = "Random-Forest",
    xgbTree = "XGBoost",
    pcaNNet = 'Neural-Networks',
    svmPoly = 'SVM'
    
  )
  
  cat("\n Those are the methods you can use:\n", unlist(method_names))
  
  
  methods_singlelearner <- c('') # This has not been implemented
  
  methods_twolearner <-
    methods # c(methods_twolearner, methods_singlelearner) # Putting all together  (methods_bilearner (Heckmann))
  
  # This is the formula that will be used in algorithms
  common_formula <- paste(controls, collapse = " + ")
  
  # And the one used for glmnet with expansions
  j = length(controls)
  common_formula_inter <- common_formula
  for (control in controls) {
   # print(control)
    for (element in controls[(length(controls) - j):length(controls)]) {
      inter <- paste(control, element, sep = ":")
    }
    common_formula_inter <- paste(common_formula, inter, sep = ' + ')
    
  }
  
  # Assign function from sourced methods_function.R to two learners or single learners
  x <- list()
  fit_ML <- list()
  
  for (method in methods) {
    x[[method]] <- common_formula
    if (method %in% c('glmnet_exp')) {
      x[[method]] <- common_formula_inter
    }
  }
  
  
  for (method in methods) {
    if (method %in% methods_twolearner) {
      # if the outcome is binary, use the two_learner_binary_function, otherwise use the two_learner_continuous_function
      fit_ML[[method]] <-
        ifelse(
          length(unique(data[[y]])) == 2,
          two_learner_binary_function,
          two_learner_continuous_function
        )
    }
    
    if (method %in% methods_singlelearner) {
      fit_ML[[method]] <- single_learner_function
    }
  }
  
  #-------------------------------------------------------
  # 2.2 Tuning parameters for ML models
  #-------------------------------------------------------
  
  # THey are saved in a config file 
  source("codes/0_config.R")
  
  # Save tunning parameter
  # They will be saved in the settings folder
  save(tune_param,
       file = paste(path_list$settings_folder, 'tuning_parameters', sep = '/'))
  
  
  
  #########################################################
  # 3. CONSTRUCT ML PROXIES
  #########################################################
  
  #-------------------------------------------------------
  # 3.1 Loop to construct ML proxy predictors (THIS IS LONG)
  #-------------------------------------------------------
  
  start.time <- Sys.time()
  foreach(
    t = 1:sim,
    .combine = 'cbind',
    .inorder = TRUE,
    .packages = vec.pac
  ) %dopar% {
    source("codes/2b_methods_functions.R")
    source("codes/0_helper_functions.R")
    print(t)
    set.seed(t)
    aux_sample <-
      auxil[[K * (t - 1) + 1]]# We look for observations in the auxiliarry sample
    datause <- data[aux_sample, ]
    dataout <- data[-aux_sample, ]
    ind_u <-
      which(datause[, 'treatment'] == 1) # treated individuals in the auxiliary sample
    
    for (method in methods) {
      print(method)
      if (run_ml == TRUE) {
        # For each replication and each method:
        ## Machine learning algorithms are trained on the auxiliary sample
        ## Then they are evaluated on the main sample to get B and S proxies, which are saved
        ## Classical ML performance metrics (AUC or MAE) are also saved for the following prediction tasks:
        ##     - predicting Y for control units in the AUX sample
        ##     - predicting Y for treated units in the AUX sample
        ##     - predicting Y for control units in the MAIN sample
        ##     - predicting Y for treated units in the MAIN sample
        ## The hyperparameters of the final model (obtained after tuning) are finally saved.
        
        #We run the algorithm on the data with tuning params set earlier
        fit_ML_results <-
          fit_ML[[method]](method,
                           x,
                           tune_param,
                           y,
                           datause,
                           dataout,
                           ind_u,
                           path_list,
                           name_BS_file,
                           t,
                           d)
        # Collect results
        split_t_results_to_save <-
          list(
            id = fit_ML_results$id,
            B = fit_ML_results$B,
            S = fit_ML_results$S,
            sim = t,
            method = method,
            perf_0 = fit_ML_results$perf_aux_main_0,
            perf_1 = fit_ML_results$perf_aux_main_1
          )
        
        # Save computes B and S 
        save(split_t_results_to_save, file = name_BS_file(method, t, path_list))
        
        # Save best model hyperparameters
        besttune_to_save <-
          list(besttune_0 = fit_ML_results$bestTune_0,
               besttune_1 = fit_ML_results$bestTune_1)
        save(besttune_to_save, file = name_bestTune_file(method, t, path_list))
      }
    }
    
    end.time <- Sys.time()
    tot.time <- end.time - start.time
    cat('Estimations were made in :')
    print(tot.time)
    cat('seconds')
    
  }
  
  
  #########################################################
  # 4. REGRESSIONS
  #########################################################
  
  #-------------------------------------------------------
  # 4.1 Recover all saved B and S
  #-------------------------------------------------------
  
  # Here we recover B and S computed during section 2
  # This is made in order to be able to get access to results without having to rerun all ML algorithms

  all_B_and_S <-
    foreach(
      t = 1:sim,
      .combine = 'cbind',
      .inorder = FALSE,
      .packages = vec.pac
    ) %dopar% {
      #all_B_and_S <- foreach(t = 1:sim, .combine='cbind', .inorder=FALSE, .packages=vec.pac) %do% {
      set.seed(t)
      B_and_S_for_each_method <- list()
      
      aux_sample <- auxil[[K * (t - 1) + 1]]
      datause <- data[aux_sample,]
      dataout <- data[-aux_sample,]
      ind_u <- which(datause[, 'treatment'] == 1)
      
      for (method in methods) {
        print(method)
        tryCatch({
          load(name_BS_file(method, t, path_list))
        },
        error = function() {
          message("You did not compute the ML estimation ! ")
          return(NA)
        })
        
        
        B_and_S <- list(
          id = as.numeric(split_t_results_to_save$id),
          method = as.character(split_t_results_to_save$method),
          sim = as.numeric(split_t_results_to_save$sim),
          B = split_t_results_to_save$B,
          S = split_t_results_to_save$S,
          perf_0 = split_t_results_to_save$perf_0,
          perf_1 = split_t_results_to_save$perf_1
        )
        
        B_and_S_for_each_method[[method]] <- B_and_S
      }
      all_B_and_S <- B_and_S_for_each_method
      
    }
  
  #-------------------------------------------------------
  # 4.2 Prepare data for regressions
  #-------------------------------------------------------
  
  # Quantiles
  G1 <-
    paste("G", 1, sep = "") # First Quantile / Least affected group for positive impacts - added to generalize
  Gp <-
    paste("G", p, sep = "") # Last Quantile / Most affected group for positive impacts - added to generalize
  all_quantiles_names <- paste("G", as.character(1:p), sep = "")
  
  # CLAN variables are the ones in the dataset that finish by "_c" , CLAN variables do not have to be the same as the ones used for the estimation of CATE with algorithms
  CLAN_var <- colnames(data %>% dplyr::select(ends_with("_c")))
  
  # You need to tranform categorical variables into dummies
  categ_var = c()
  bin_var = c()
  cont_var = c()
  for (covariate in CLAN_var) {
    # cat(covariate)
    # cat(": ")
    if (length(unique(data[, covariate])) == 2) {
      bin_var <- c(bin_var, covariate)
     # cat(' \n This is a binary variable \n')
    }
    if (length(unique(data[, covariate])) > 2 &
        length(unique(data[, covariate])) < 10) {
      categ_var <- c(categ_var, covariate)
     # cat('\n This is a continuous variable \n')
      
    }
    if (length(unique(data[, covariate])) >= 10) {
      cont_var <- c(cont_var, covariate)
     # cat('\n This is a continuous variable \n')
    }
    
  }
  
  
  ### This function transforms categorical variable of a dataframe into numeric
  ### Input :  dataframe, name of categorical variables column
  ### Output  :  categorical transformed variables
  
  bin_var_as_dummies <- categorical_to_discrete(data,  bin_var)
  #categ_var_as_dummies <- categorical_to_discrete(data, categ_var)
  data <- cbind(data, bin_var_as_dummies)
  #data <- cbind(data, categ_var_as_dummies)
  
  # Variables used for CLAN
  affected.dummies.or.continuous <-
    c(cont_var, colnames(bin_var_as_dummies)) #categ_var_as_dummies
  
  
  #-------------------------------------------------------
  # 4.3 Regression loop
  #-------------------------------------------------------
r <- NULL
  # r <-
  #   foreach(
  #     t = 1:sim,
  #     .combine = 'cbind',
  #     .inorder = TRUE,
  #     .packages = vec.pac
  #   ) %dopar% {
for(t in c(1:sim)){
      set.seed(t)
      # seed must be set inside the foreach loop, otherwise it varies
      
      # Prepare empty results tables
      results_BLP   <- list()
      results_GATES <- list()
      results_CLAN  <- list()
      results_GROUP <- list()
      bestML        <- list()
      
      # Create the auxiliary and main samples
      # - using the corresponding sampled observations for a given t
      ind <-
        auxil[[K * (t - 1) + 1]] # only if K = 2 - note that for a different K,
      # we may not use createMultiFolds wich creates folds with common elements
      # when K =/= 2
      # Note: we removed the complete.cases function since data should already be complete
      
      datause <- data[ind, ]
      dataout <- data[-ind, ]
      ind_u <-
        which(datause[, 'treatment'] == 1)         # treatment indicator
      
      ### Propensity score
      ## Use the propensity score to compute the weights ((1-p)*p)^-1 for GML
      dataout['prop']  <-
        rep((nrow(datause[datause[, 'treatment'] == 1, ]) + nrow(dataout[dataout[, 'treatment'] ==
                                                                           1, ])) / (nrow(datause) + nrow(dataout)), nrow(dataout))
      
      for (method in methods) {
        print(method)
        
        
        #Get B and S
        B <- all_B_and_S[[method, t]]$B
        S <- all_B_and_S[[method, t]]$S
        id <- all_B_and_S[[method, t]]$id
        perf_0 <- all_B_and_S[[method, t]]$perf_0
        perf_1 <- all_B_and_S[[method, t]]$perf_1
        
        
        S2        <- S + runif(length(S), 0, 0.00001)
        breaks    <- quantile(S2, seq(0, 1, 1 / p),  include.lowest = T)
        breaks[1] <- breaks[1] - 0.001
        breaks[p + 1] <- breaks[p + 1] + 0.001
        SG        <- cut(S2, breaks = breaks)
        
        SGX       <- model.matrix( ~ -1 + SG)
        DSG       <-
          data.frame(as.numeric(I(as.numeric(dataout[, 'treatment']) - dataout[, "prop"])) *
                       SGX)
        
        colnames(DSG) <- all_quantiles_names
        
        dataout[, c("B", "S", colnames(DSG), "weight")] <-
          cbind(B, S, DSG, as.numeric((1 / (
            dataout[, "prop"] * (1 - dataout[, "prop"])
          ))))
        
        # BP modification : add noise in case prediction cst
        # so that in the regression phase, no collinearity between
        # B and S, so that no NA in estimated coef
        
        # Bruit si pas de variance ou nombre de valeurs tr?s faible
        if (var(dataout$B) == 0) {
          dataout$B <- dataout$B + rnorm(length(dataout$B),  mean = 0, sd = 0.1)
        }
        if (var(dataout$S) == 0) {
          dataout$S <- dataout$S + rnorm(length(dataout$S),  mean = 0, sd = 0.1)
        }
        
        ############################################# ATE by groups #############################################
        
        
        # Compute the GATES estimates
        
        # 1. Run regression
        # {
        sum_groups <-
          paste(all_quantiles_names, collapse = "+") # modified to generalized for any p (nb quantiles/groups)
        form1 <-
          as.formula(paste(
            paste(y, "~", "B+S+", sum_groups, sep = ""),
            " | ",
            fixed_effect,
            "| 0 |",
            cluster,
            sep = ''
          ))
        
        reg <- tryCatch({
          reg <- felm(form1, data = dataout, weights = dataout$weight)
          
        }, error = function(e) {
          cat("ERROR :", method, t, "\n")
          
          form1 <-
            as.formula(paste(
              paste(y, "~", "S+", sum_groups, sep = ""),
              " | ",
              fixed_effect,
              "| 0 |",
              cluster,
              sep = ''
            ))
          reg    <- felm(form1, data = dataout, weights = dataout$weight)
          
        }, warning = function(war) {
          cat("WARNING :", method, t, "\n")
          form1  <-
            as.formula(paste(
              paste(y, "~", "S+", sum_groups, sep = ""),
              " | ",
              fixed_effect,
              "| 0 |",
              cluster,
              sep = ''
            ))
          reg    <- felm(form1, data = dataout, weights = dataout$weight)
          
        })
        
        
        # 2. Save results: coef, ICleft, IC right, pval
        add_to_results <- list()
        for (quantile in all_quantiles_names) {
          add_to_results[['coef']] <-
            summary(reg)$coefficients[quantile, "Estimate"]
          add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left' , sep =
                                  '')]] <- confint(reg, quantile, level = 1 - alpha)[1]
          add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right' , sep =
                                  '')]] <- confint(reg, quantile, level = 1 - alpha)[2]
          add_to_results[['stderr']] <-
            summary(reg)$coefficients[quantile, 2]
          add_to_results[['pval']] <-
            summary(reg)$coefficients[quantile, "Pr(>|t|)"]
          results_GATES[[method]][['GATES']][[quantile]] <-
            add_to_results
          
        }
        
        #3. Test the difference of treatment effects between the most affected group and least affected group
        
        # Difference between least and most (not sorted)
        test <-
          glht(reg, linfct = c(paste(paste(Gp, G1, sep = "-"), "==0", sep = "")))
        pval <- summary(test)$test$pvalues[1]
        
        # Add results of tests (least and most + all)
        add_to_results <- list()
        simultaneous_confint <- confint(test, level = 1 - alpha)
        add_to_results[["coef"]]  <-
          simultaneous_confint$confint[, "Estimate"]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left' , sep = '')]]  <-
          simultaneous_confint$confint[, "lwr"]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right' , sep = '')]]  <-
          simultaneous_confint$confint[, "upr"]
        add_to_results[['pval']] <- pval
        results_GATES[[method]][['GATES']][['test_diff']] <-
          add_to_results
        
        
        #2bis. Monotonized GATES (use this only in the GATES plot, just as Chernozhukov et al).
        # Save results: coef, ICleft, IC right:
        
        mean <- summary(reg)$coef[colnames(DSG), 1]
        sd   <- summary(reg)$coef[colnames(DSG), 2]
        order <- names(sort(mean)) # monotonized groups
        
        # crit = quantile associated to the proba 1-alpha/(p)
        # ie crit is s.t.: P(X<=crit) = 1- alpha/(p) where X ~ normal distribution(0,1)
        crit <- qnorm(1 - alpha / (p))
        
        add_to_results <- list()
        for (g in 1:p) {
          add_to_results[['coef']] <- sort(mean)[[g]]
          add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left' , sep =
                                  '')]] <- sort(mean - crit * sd)[[g]]
          add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right' , sep =
                                  '')]] <- sort(mean + crit * sd)[[g]]
          add_to_results[['stderr']] <-
            summary(reg)$coefficients[order[g], 2]
          add_to_results[['pval']] <-
            summary(reg)$coefficients[order[g], "Pr(>|t|)"]
          results_GROUP[[method]][['monotonized(GATES)']][[paste0("G", g)]] <-
            add_to_results
        }
        
        #3bis. Test the difference of treatment effects between the (sorted) most affected group and (sorted) least affected group
        
        
        # Difference between least and most (sorted)
        test <-
          glht(reg, linfct = c(paste(
            paste(order[p], order[1], sep = "-"), "==0", sep = ""
          )))
        pval <- summary(test)$test$pvalues[1]
        
        # Add results of tests (least and most + all)
        add_to_results <- list()
        simultaneous_confint <- confint(test, level = 1 - alpha)
        add_to_results[["coef"]]  <-
          simultaneous_confint$confint[, "Estimate"]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left' , sep = '')]]  <-
          simultaneous_confint$confint[, "lwr"]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right' , sep = '')]]  <-
          simultaneous_confint$confint[, "upr"]
        add_to_results[['pval']] <- pval
        results_GROUP[[method]][['monotonized(GATES)']][['test_diff']] <-
          add_to_results
        
        
        # Test all (not needed here)
        # test_all <- glht(reg, linfct = R,rhs=c(0,0,0))
        # pval_all <- summary(test_all, test=Ftest())$test$pvalue[1]
        # add_to_results <- list()
        # add_to_results[['pval']] <- pval_all
        # results_GATES[[method]][['GATES']][['test_diff_all']] <- add_to_results
        
        bestML[[method]] <- list(Lambda = NULL, Bar_Lambda = NULL)
        bestML[[method]][['Bar_Lambda']]  <- sum(mean ^ 2) / 5
        # }
        
        
        ################################### Best Linear Prediction Regression  ###################################
        
        
        # Estimate beta1 (ATE) and beta2 (heterogeneity)
        Sd            <- dataout$S - mean(dataout$S)
        dataout$S_ort <-
          I((as.numeric(dataout[, 'treatment']) - dataout[, "prop"]) * Sd)
        dataout$d_ort <-
          I((as.numeric(dataout[, 'treatment']) - dataout[, "prop"]))
        
        form1 <-
          as.formula(paste(
            paste(y, "~", "B+S+d_ort+S_ort", sep = ""),
            " | ",
            fixed_effect,
            "| 0 |",
            cluster,
            sep = ''
          ))
        
        
        reg <- tryCatch({
          reg <- felm(form1, data = dataout, weights = dataout$weight)
          
        }, error = function(e) {
          cat("ERROR :", method, t, "\n")
          form1 <-
            as.formula(paste(
              paste(y, "~", "d_ort+S_ort", sep = ""),
              " | ",
              fixed_effect,
              "| 0 |",
              cluster,
              sep = ''
            ))
          reg    <- felm(form1, data = dataout, weights = dataout$weight)
          
          
        }, warning = function(war) {
          cat("WARNING :", method, t, "\n")
          form1 <-
            as.formula(paste(
              paste(y, "~", "d_ort+S_ort", sep = ""),
              " | ",
              fixed_effect,
              "| 0 |",
              cluster,
              sep = ''
            ))
          reg    <- felm(form1, data = dataout, weights = dataout$weight)
          
          
        })
        
        
        coef <- (summary(reg)$coefficients['d_ort', 1])
        stderr <- (summary(reg)$coefficients['d_ort', 2])
        pval <- (summary(reg)$coefficients['d_ort', 4])
        add_to_results <- list()
        add_to_results[['coef']] <- coef
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = '')]] <-
          confint(reg, 'd_ort', level = 1 - alpha)[1]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = '')]] <-
          confint(reg, 'd_ort', level = 1 - alpha)[2]
        add_to_results[['stderr']] <- stderr
        add_to_results[['pval']] <- pval
        results_BLP[[method]][['BLP']][['ATE']] <- add_to_results
        
        coef <- (summary(reg)$coefficients['S_ort', 1])
        stderr <- (summary(reg)$coefficients['S_ort', 2])
        pval <- (summary(reg)$coefficients['S_ort', 4])
        add_to_results <- list()
        add_to_results[['coef']] <- coef
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = '')]] <-
          confint(reg, 'S_ort', level = 1 - alpha)[1]
        add_to_results[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = '')]] <-
          confint(reg, 'S_ort', level = 1 - alpha)[2]
        add_to_results[['stderr']] <- stderr
        add_to_results[['pval']] <- pval
        results_BLP[[method]][['BLP']][['HET']] <- add_to_results
        
        bestML[[method]][['Lambda']]      <-
          abs(summary(reg)$coefficients['S_ort', 1]) * sqrt(var(dataout$S))
        bestML[[method]][['Perf_Aux_0']]  <- perf_0[1]
        bestML[[method]][['Perf_Aux_1']]  <- perf_1[1]
        bestML[[method]][['Perf_Main_0']]  <- perf_0[2]
        bestML[[method]][['Perf_Main_1']]  <- perf_1[2]
        
        
        ################################################ CLAN with Most/Least Affected  ################################################
        
        
        high.effect     <- quantile(dataout$S, 1 - thres)
        
        low.effect      <- quantile(dataout$S, thres)
        
        dataout$h       <- as.numeric(dataout$S > high.effect)
        dataout$l       <- as.numeric(dataout$S < low.effect)
        
        if (var(dataout$h) == 0) {
          dataout$h <- as.numeric(runif(length(dataout$h)) < 0.1)
        }
        if (var(dataout$l) == 0) {
          dataout$l <- as.numeric(runif(length(dataout$l)) < 0.1)
        }
        
        for (characteristic in affected.dummies.or.continuous) {
          #print(characteristic)
          #characteristic = "b_Prop_epargne_formel"
          #characteristic= "b_j8_patience_scale"
          
          if (var(dataout[(dataout$h == 1) |
                          (dataout$l == 1), characteristic], na.rm = TRUE) == 0) {
            cat("Zero variance for variable ")
            cat(characteristic)
            cat(" in iteration ")
            cat(t)
            cat("\n")
            # whenever there is no variance anymore in the main sample
            # for one variable of the CLAN, replace the variable by some noise
            # so that the code does not return any error
            dataout[, characteristic] <-
              rnorm(n = dim(dataout)[1], 0, 0.01)
          }
          
          a  <- tryCatch({
            form  <- paste(characteristic, "~h+l-1", sep = "")
            reg   <-
              lm(form, data = dataout[(dataout$h == 1) | (dataout$l == 1), ])
            coef  <- reg$coefficients['h'] - reg$coefficients['l']
            test  <- glht(reg, linfct = c("h-l==0"))
            
            res1 <- list()
            res1[['coef']]  <- summary(reg)$coefficients['h', 1]
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              confint(reg, 'h', level = 1 - alpha)[1]
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              confint(reg, 'h', level = 1 - alpha)[2]
            res1[['pval']] <- summary(reg)$coefficients['h', 4]
            
            res2 <- list()
            res2[['coef']]  <- summary(reg)$coefficients['l', 1]
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]] <-
              confint(reg, 'l', level = 1 - alpha)[1]
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]] <-
              confint(reg, 'l', level = 1 - alpha)[2]
            res2[['pval']] <- summary(reg)$coefficients['l', 4]
            
            res3 <- list()
            simultaneous_confint <- confint(test, level = 1 - alpha)
            res3[["coef"]]  <- simultaneous_confint$confint[, "Estimate"]
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              simultaneous_confint$confint[, "lwr"]
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              simultaneous_confint$confint[, "upr"]
            res3[['pval']] <- summary(test)$test$pvalues[1]
            
            a     <-
              list('Gp_res' = res1,
                   'G1_res' = res2,
                   'Diff' = res3)
            
          }, error = function(e) {
            cat("ERROR3 :", method, t, "\n")
            
            
            res1 <- list()
            res1[['coef']]  <-
              mean(dataout[(dataout$h == 1), characteristic], na.rm = T)
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              mean(dataout[(dataout$h == 1), characteristic], na.rm = T)
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              mean(dataout[(dataout$h == 1), characteristic], na.rm = T)
            res1[['pval']] <- 0
            
            res2 <- list()
            res2[['coef']]  <-
              mean(dataout[(dataout$l == 1), characteristic], na.rm = T)
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]] <-
              mean(dataout[(dataout$l == 1), characteristic], na.rm = T)
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]] <-
              mean(dataout[(dataout$l == 1), characteristic], na.rm = T)
            res2[['pval']] <- 0
            
            res3 <- list()
            simultaneous_confint <- confint(test, level = 1 - alpha)
            res3[["coef"]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[['pval']] <- 0
            
            a     <-
              list('Gp_res' = res1,
                   'G1_res' = res2,
                   'Diff' = res3)
            
          }, warning = function(cond) {
            cat("ERROR2 :", method, t, "\n")
            res1 <- list()
            res1[['coef']]  <-
              mean(dataout[(dataout$h == 1), characteristic])
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              mean(dataout[(dataout$h == 1), characteristic])
            res1[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              mean(dataout[(dataout$h == 1), characteristic])
            res1[['pval']] <- 0
            
            res2 <- list()
            res2[['coef']]  <-
              mean(dataout[(dataout$l == 1), characteristic])
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]] <-
              mean(dataout[(dataout$l == 1), characteristic])
            res2[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]] <-
              mean(dataout[(dataout$l == 1), characteristic])
            res2[['pval']] <- 0
            
            res3 <- list()
            simultaneous_confint <- confint(test, level = 1 - alpha)
            res3[["coef"]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'left', sep = "")]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[[paste('CI_lvl', (1 - alpha) * 100, 'right', sep = "")]]  <-
              tryCatch({
                res1['coef'] - res2['coef']
              }, error = function(e) {
                return(NA)
              })
            res3[['pval']] <- 0
            
            a     <-
              list('Gp_res' = res1,
                   'G1_res' = res2,
                   'Diff' = res3)
            
            
          })
          results_CLAN[[method]][[characteristic]]   <- a
          
        }
        
      } # end loop methods
      
      
      r <- cbind(r, list('BLP' = results_BLP,
                         "GROUP" = results_GROUP ,
                         "GATES" = results_GATES,
                         'CLAN' = results_CLAN,
                         'PERF' = bestML))
      
      
      # r <- list(
      #   'BLP' = results_BLP,
      #   "GROUP" = results_GROUP ,
      #   "GATES" = results_GATES,
      #   'CLAN' = results_CLAN,
      #   'PERF' = bestML
      # )
      
    }  # end loop foreach
  
  # r
  
  #-------------------------------------------------------
  # 3.5 Aggregate results into table
  #-------------------------------------------------------
  
  options(scipen = 999)
  
  # Get all results (BLP, GATES, CLAN) in
  all_results <-
    by(r, seq_len(nrow(r)), function(row) {
      retrieve_results(row)
    })
  output <-
    data.frame(do.call("rbind", all_results[-length(all_results)]))
  output <- output[, c(2:ncol(output))]
  colnames(output) <- c("method", "type", "var", "elements", "value")
  
  # Performance metric table
  perf <- data.frame(all_results[length(all_results)])
  perf <- perf[, c(2:ncol(perf))]
  colnames(perf) <- c("method", "type", "value")
  perf_table <- spread(perf, type, value)
  
  # P-values adjustment
  output <- output[!is.na(output$var), ]
  output[output$elements == "pval", ]$value <-
    output[output$elements == "pval", ]$value * 2
  # Check if any rows meet the conditions
  if (nrow(output[(output$elements == "pval") & (output$value >= 1), ]) > 0) {
    # If the subset is not empty, perform the assignment
    output[(output$elements == "pval") & (output$value >= 1), ]$value <- 1
  }
  
  # Put into pretty table
  output <- spread(output, elements, value)
  colnames(output) <-
    c(
      'method',
      'type',
      'var',
      'median_ci_lower',
      'median_ci_upper',
      "median_coef",
      "median_pval",
      "median_stderr"
    )
  output <- merge(output, perf_table, by = 'method')
  
  
  # Saving part :
  
  result_path = path_list$treatment_folder
  dir.create(paste(result_path, 'output_table', sep = '/'))
  output_path = paste(result_path, 'output_table', sep = '/')
  
  file_name = paste(output_path,
                    '/results_',
                    y,
                    '_treatment',
                    treatment_arm_name,
                    '.xlsx',
                    sep = '')
  write_xlsx(output, path = file_name)
  
}
