#########################################################
## Functions to visualize HET results  
## Version : 1
## author : Morgane Hoffmann and Elia Perennes
# Last version : 06/07/2022
#########################################################


################################################################################
# 1. Function to create GATES graphs 
################################################################################

# ----------------------------------------------------------------------------
# Notes : It produces a GATES plot for each outcome of the list and a given 
#         treatment vs. control arms comparison, with ML methods as plot facets  
# 
# Parameters : outcomes : the outcomes list 
#              treatment_arms_num : the treatment arms 
#              control_arms_num : the control arms 
#              nb_gp : the number of groups (GATES)
#              path_results : the path where each outcomes results are stored
#              name : filename of the plot 
#              save : save option 
# ----------------------------------------------------------------------------

# Make sure you have these package loaded : 'dplyr', 'ggplot2', "readxl"

get_graph = function(outcomes, treatment_arms_num, control_arms_num, nb_gp, path_results, path_labels, sorted_gates = FALSE, name = "GATES", save = T){
  
  #### Install packages if it is not already done
  # vec.pac= c(
  #   'haven', 'dplyr', 'ggplot2', "readxl"
  # ) 
  # 
  # package.check <- lapply(
  #   vec.pac,
  #   FUN = function(x) {
  #     if (!require(x, character.only = TRUE)) {
  #       install.packages(x, dependencies = TRUE)
  #       library(x, character.only = TRUE)
  #     }
  #   }
  # )
  
  for (o in outcomes){

    
    # ----------------------------------------------------------------------------
    # 1. Load data 
    # ----------------------------------------------------------------------------
    
    # load variables label 
    var_labels = read_excel(path_labels)
    
    # Create Figures folder into outcome * treatment directory folder 
    path_outcome_treatment = paste0(path_results,"/",
                                    o,
                                    "/treatment_",
                                    paste0(control_arms_num, collapse = ""),
                                    "vs",
                                    paste0(treatment_arms_num, collapse = ""),
                                    "/")
    dir.create(paste0(path_outcome_treatment, "Figures/"))
    
    # Load estimates from ML estimation
    data = read_excel(paste0(path_outcome_treatment,
                             "output_table/results_",
                             o, 
                             "_treatment",
                             paste0(control_arms_num, collapse = ""),
                             "vs", 
                             paste0(treatment_arms_num, collapse = ""),
                             ".xlsx")) 
    
    data = data[,1:10] %>%  # we only need the first 10 columns 
      mutate(outcome = o)
    
    # ----------------------------------------------------------------------------
    # 2. GATES plot  
    # ----------------------------------------------------------------------------
    
    # Treatment vs controls group for plot title 
    treatment_label = paste(paste(control_arms_num,collapse=""), "vs", paste(treatment_arms_num,collapse=""), sep='')

    # Outcome for plot title
    outcome_label = var_labels$label[var_labels$variable_name==o]
    
    
    # filter data 
    if(sorted_gates){
      gates_data <- data %>% 
        filter(type == "monotonized(GATES)" & var %in% paste0("G",1:nb_gp))
      legend_name = "monotonized GATES"
    }else{
      gates_data <- data %>% 
        filter(type == "GATES" & var %in% paste0("G",1:nb_gp))
      legend_name = "GATES"
      
    }
    
    
    ate_data = data %>% 
      filter(var == "ATE")
    
    # plot 
    gates_plot = ggplot(gates_data,
                        aes(x=var, y=median_coef)) + 
      geom_point(aes(color = "Median coef."), shape = 15, size = 3)+
      geom_linerange(aes(ymin=median_ci_lower, ymax=median_ci_upper, 
                         color = "95% CI"), size = 0.75) + 
      geom_hline(data = ate_data, aes(yintercept = median_coef, size = "Median coef."), 
                 linetype = 2, color = "#1B9E77")+
      geom_hline(data = ate_data, aes(yintercept = median_ci_lower, size = "95% CI"),
                 linetype = 2, color = "#666666")+
      geom_hline(data = ate_data, aes(yintercept = median_ci_upper, size = "95% CI"),
                 linetype = 2, color = "#666666")+
      facet_wrap(~toupper(method)) +
      scale_color_manual(name = legend_name, values = c("#666666", "#1B9E77")) +
      scale_size_manual(values = c(0.5,0.5)) +
      guides(colour = guide_legend(position = "bottom", 
                                   override.aes = list(linetype = c("solid", "blank"), 
                                                       shape = c(NA, 15))), 
             size = guide_legend(title="ATE", 
                                 override.aes = list(color =  c("#666666", "#1B9E77")))) +
      theme_bw(base_size = 13)+
      labs(y = "Treatment effect", x = "Group by Heterogeneity score", 
           title = paste0("Outcome: ", outcome_label))
    
    print(gates_plot)
    
    if(save){
      # Save plot
      ggsave(gates_plot,
             filename = paste0(path_outcome_treatment, "Figures/", name, ".pdf"),
             width = 10, height = 5)
      }
    
    }
  
}


################################################################################
# 2. Function to create HET Table
################################################################################

# ----------------------------------------------------------------------------
# Notes : It produces one Table for a multiple treatments 
#         and a given treatment vs. control arms comparison, 
#         with best ML method selected for each outcome. 
# 
# Parameters : outcomes : a list of outcomes 
#              treatment_arms_num : the treatment arms 
#              control_arms_num : the control arms 
#              nb_gp : the number of groups (GATES)
#              path_results : the path where each outcome results are stored
#              path_tables : the path where each tables will be stored
#              save : save option
# ----------------------------------------------------------------------------

get_HET_table = function(outcomes, treatment_arms_num, control_arms_num, path_results, path_tables, path_labels, sorted_gates=T, save=F){
  
  dir.create(paste0(path_tables))
  
  # ----------------------------------------------------------------------------
  # 1. Load data 
  # ----------------------------------------------------------------------------
  
  # load variables label 
  var_labels = read_excel(path_labels)
  
  # load results for each outcome and aggregate all
  for (o in outcomes){
    
    # Create Tables folder into outcome * treatment directory folder 
    path_outcome_treatment = paste0(path_results,"/",
                                    o,
                                    "/treatment_",
                                    paste0(control_arms_num, collapse = ""),
                                    "vs",
                                    paste0(treatment_arms_num, collapse = ""),
                                    "/")
    
    data = read_excel(paste0(path_outcome_treatment,
                             "output_table/results_",
                             o, 
                             "_treatment",
                             paste0(control_arms_num, collapse = ""),
                             "vs", 
                             paste0(treatment_arms_num, collapse = ""),
                             ".xlsx"))
    
    data = data[,1:10] %>%  # we only need the first 10 columns 
      mutate(outcome = o)
    
    # Load estimates from ML estimation
    assign(paste0("data_table_", o), data, envir = .GlobalEnv) 
    
    rm("data") 
  }
  
  dfs = lapply(ls(pattern="data_table_", envir = .GlobalEnv), function(x) get(x))
  data_table_all = rbindlist(dfs)
  
  # ----------------------------------------------------------------------------
  # 2 Data preparation
  # ----------------------------------------------------------------------------
  
  # filter data 
  if(sorted_gates){
    type_gates = "monotonized(GATES)"
  }else{
    type_gates = "GATES"
  }
  
  tmp = data_table_all %>%
    group_by(outcome) %>%
    mutate(best_ML = Lambda == max(Lambda)) %>%
    ungroup() %>%
    filter(best_ML & 
             (type == "BLP" | (type == type_gates & (var == "test_diff" | var == "G1" | var == "G5")))) %>%
    mutate(median_coef_round = round(median_coef, 3), 
           signif_10pct = median_pval < 0.1,
           signif_5pct = median_pval < 0.05,
           signif_1pct = median_pval < 0.01, 
           median_coef_round_stars = as.character(ifelse(signif_1pct, paste0(median_coef_round,"***"),
                                                         ifelse(signif_5pct, paste0(median_coef_round,"**"),
                                                                ifelse(signif_10pct, paste0(median_coef_round,"*"),
                                                                       median_coef_round))))) %>%
    dplyr::select(outcome, type, var, median_coef_round_stars, method)
  
  # ----------------------------------------------------------------------------
  # 3. Table 
  # ----------------------------------------------------------------------------
  
  get_outcome_row = function(data, o){
    
    BLP_ATE = data %>% filter(outcome == o & type == "BLP" & var == "ATE") %>% dplyr::select(median_coef_round_stars) %>% as.character()
    BLP_HET = data %>% filter(outcome == o & type == "BLP" & var == "HET") %>% dplyr::select(median_coef_round_stars) %>% as.character()
    GATES_G1 = data %>% filter(outcome == o & type == type_gates & var == "G1") %>% dplyr::select(median_coef_round_stars) %>% as.character()
    GATES_G3 = data %>% filter(outcome == o & type == type_gates & var == "G5") %>% dplyr::select(median_coef_round_stars) %>% as.character()
    GATES_diff = data %>% filter(outcome == o & type == type_gates & var == "test_diff") %>% dplyr::select(median_coef_round_stars) %>% as.character()
    method = data %>% filter(outcome == o) %>% dplyr::select(method) %>% unique()
    
    return(as.character(c(o,BLP_ATE, BLP_HET, GATES_G1, GATES_G3, GATES_diff, method)))
    
  }
  
  data_table = get_outcome_row(data = tmp, o = outcomes[1]) %>% t() %>% as.data.frame()
  
  if(length(outcomes)>1){
    for(o in outcomes[-1]){
      data_table = rbind(data_table, 
                         get_outcome_row(data = tmp, o = o))
    }
  }
  
  
  colnames(data_table) = c("variable_name", "Average intent to treat", "Heterogeneity", "Least affected", "Most affected", "(Most - Least)", "Machine Learning method")
  row.names(data_table) = NULL
  data_table = left_join(data_table, var_labels) %>% 
    dplyr::select(-variable_name) %>% 
    rename("Outcome" = label) %>% 
    relocate(Outcome)
  
  
  table = data_table %>%
    flextable() %>%
    theme_booktabs(bold_header = TRUE) %>%
    set_table_properties(layout = "autofit", width = 1) %>%
    align(align = "center", part = "all") %>%
    add_header_row(
      values = c(" ", "Best linear predictor", "Group average intent to treat", " "),
      colwidths = c(1, 2, 3, 1)) 
  
  table %>%
    flextable_to_rmd()
  
  
  if(save){
  
    save_as_docx(table = paste(paste(control_arms_num,collapse=""), "vs", paste(treatment_arms_num,collapse=""), sep=''),
                 path = paste0(path_tables, "/heterogeneity_results_",
                               paste0(paste0("T", treatment_arms_num, collapse = "&"),
                                      "vs",
                                      paste0("T", control_arms_num, collapse = "&")),
                               ".docx"))
    }
  
}


################################################################################
# 3. Function to create CLAN Table
################################################################################

# ----------------------------------------------------------------------------
# Notes : 
# 
# Parameters : 
# ----------------------------------------------------------------------------


get_CLAN_table = function(outcomes, treatment_arms_num, control_arms_num, path_results, path_tables, path_labels, save=F){
  
  dir.create(paste0(path_tables))
  
  
  # ----------------------------------------------------------------------------
  # 1. Load data 
  # ----------------------------------------------------------------------------
  
  var_labels = read_excel(path_labels)
  
  for (o in outcomes){
    
    # Create Tables folder into outcome * treatment directory folder 
    path_outcome_treatment = paste0(path_results,"/",
                                    o,
                                    "/treatment_",
                                    paste0(control_arms_num, collapse = ""),
                                    "vs",
                                    paste0(treatment_arms_num, collapse = ""),
                                    "/")
    
    data = read_excel(paste0(path_outcome_treatment,
                             "output_table/results_",
                             o, 
                             "_treatment",
                             paste0(control_arms_num, collapse = ""),
                             "vs", 
                             paste0(treatment_arms_num, collapse = ""),
                             ".xlsx"))
    
    data = data[,1:10] %>%  # we only need the first 10 columns 
      mutate(outcome = o)
    
    
    # Load estimates from ML estimation
    assign(paste0("data_table_", o), data, envir = .GlobalEnv)
    
    rm("data") 
  }
  
  dfs = lapply(ls(pattern="data_table_", envir = .GlobalEnv), function(x) get(x))
  data_table_all = rbindlist(dfs)
  
  # ----------------------------------------------------------------------------
  # 2 Data preparation
  # ----------------------------------------------------------------------------
  
  tmp = data_table_all %>%
    group_by(outcome) %>%
    mutate(best_ML = Lambda == max(Lambda),  
           HET_signif = max(var == "HET" & median_pval <0.11)) %>% # we restrict to outcomes for which we detect heterogeneity 
    ungroup() %>%
    filter(best_ML & HET_signif & 
             (var == "Diff" | var == "G1_res" | var == "Gp_res")) %>%
    mutate(median_coef_round = round(median_coef, 3), 
           signif_10pct = median_pval < 0.1,
           signif_5pct = median_pval < 0.05,
           signif_1pct = median_pval < 0.01, 
           median_coef_round_stars = as.character(ifelse(signif_1pct, paste0(median_coef_round,"***"),
                                                         ifelse(signif_5pct, paste0(median_coef_round,"**"),
                                                                ifelse(signif_10pct, paste0(median_coef_round,"*"),
                                                                       median_coef_round))))) %>%
    dplyr::select(outcome, type, var, median_coef_round_stars, method)
  
  tmp_wide = spread(tmp, var, median_coef_round_stars)
  
  # ----------------------------------------------------------------------------
  # 3 Print and save tables 
  # ----------------------------------------------------------------------------
  
  outcomes_signif = unique(tmp_wide$outcome)
  
  for (o_signif in outcomes_signif){
    
    data_table = tmp_wide %>% filter(outcome == o_signif) %>% dplyr::select(type, Gp_res, G1_res, Diff) %>% as.data.frame()
    data_table$type = gsub("_1", "", data_table$type)
    
    data_table = left_join(data_table, var_labels, by = c("type"="variable_name")) %>% dplyr::select(label, Gp_res, G1_res, Diff)
    colnames(data_table) = c("Variable", "Most affected", "Least affected", "Difference")
    row.names(data_table) = NULL
    
    outcome_lab = var_labels$label[var_labels$variable_name==o_signif]
    table = data_table %>%
      flextable() %>%
      add_header_lines(paste0("Outcome: ", outcome_lab)) %>%
      theme_booktabs(bold_header = TRUE) %>%
      set_table_properties(layout = "autofit", width = 1) %>%
      align(align = "center", part = "all") %>%
      align(align = "left", j = 1)
    
    table %>%
      flextable_to_rmd()
    
    if(save){
      
      save_as_docx(table = paste(paste(control_arms_num,collapse=""), "vs", paste(treatment_arms_num,collapse=""), sep=''),
                   path = paste0(path_tables, "/CLAN_results_",
                                 o_signif,"_",
                                 paste0(paste0("T", treatment_arms_num, collapse = "&"),
                                        "vs",
                                        paste0("T", control_arms_num, collapse = "&")),
                                 ".docx"))
    }
    
  }
  
}

