########################################################################
## Heterogeneous treatment effects >Djibouti
## Version : 2
## author : Morgane Hoffmann and Elia Perennes
# Last version : 03/11/2023
#######################################################################

# -----------------------------------------------------------------------#
# 0. INITIALIZATION
# 0.A Load packages
# 0.B Import custom functions
# 0.C Folder structure and working directories

# 1. DATA CLEANING

# 2. GML ESTIMATION
# 2.A Set parameters
# 2.B Run Generic Machine Learning

# 3. RESULTS VISUALIZATION
# -----------------------------------------------------------------------#



#########################################################
# 0. INITIALIZATION
#########################################################
# Start from a fresh environment

# .rs.restartR()       # Restart R session for full refresh
rm(list = ls())

# please restore the environment the first time you interact with this package
# by uncommenting the following line:

#renv::restore()

#-------------------------------------------------------
# 0.A Load packages
#-------------------------------------------------------

# Load and install packages
source("codes/0_install_packages.R")

#-------------------------------------------------------
# 0.B Import custom functions
#-------------------------------------------------------

# Import helper functions
source("codes/0_helper_functions.R")

# main estimation function for Generic Machine Learning
source("codes/2_gml_function.R")

# Import algorithms functions
source("codes/2b_methods_functions.R")

# Import chart functions
source("codes/3_visualization_functions.R")

#-------------------------------------------------------
# 0.C Folder structure and working directories
#-------------------------------------------------------

# The folder architecture for results will be the following :
# Generic Machine learning R project
#  __codes n
#    __main.R .. [main.R and all programs needed, see Readme file]
#  __data [all data files needed, see Readme file]
#    __datafile.RData 
#  __results
#   __experimentname
#     __"outcome1"  # Each outcome have its own file where we save all ML predictions and BLP, GATES and CLAN results
#     __"outcome2"
#       __"treatmentivstreamentj" # Each types of treatment tested have its own file (if you only have T1 vs T0 you will have only one file here)
#         __B_and_S  # These are the B and S estimations obtained from each ML method, for each replication
#         __bestTune # These are the hyperparameters of the best model for each ML method, for each replication
#         __output_tables # This file contains BLP, GATES and CLAN results that are gathered in one unique csv file
#         __sampled # We save here the ids of observations in the auxiliary and main samples
#         __settings # We also keep here the tuning parameters of the ML algorithms, you can always access it in case you forgot which one you put


cat("This is your working directory : ", getwd())

# Create results directory if it does not exist
if (!dir.exists('results')) {
  dir.create("results")
}
if (!dir.exists('results')) {
  print('No data file')
}


#########################################################
# 1. DATA CLEANING
#########################################################

# This is the first folder created in results "experiment" where results will be stored
# Change the name of the experiment according to which data you want to have results for
name_of_experiment = 'DjiboutiMidline'
  #'DjiboutiMidline' #DjiboutiEndline 

# Data will be cleaned and saved in the data "folder" calling the .R file
if (name_of_experiment == 'DjiboutiFollowUp') {
  source("codes/1_clean_djibouti.R")
  #data_name = "djibouti_followup"
  
} else if (name_of_experiment == 'DjiboutiEndline') {
  source("codes/1_clean_djibouti_endline.R")
  #data_name = "djibouti_endline"
  
} else if (name_of_experiment == 'DjiboutiMidline') {
  source("codes/1_clean_djibouti_midline.R")
  #data_name = "djibouti_midline"
} else{
  
}


# If you have new data to add that is not Endline, Midline or Followup then run the following
#source("codes/clean_my_new_data.R")
# Don't forget to have in memory the name of your file
#data_name = "my_file_name"

#########################################################
# 2. GML ESTIMATION
#########################################################

#-------------------------------------------------------
# 2.A Set parameters
#-------------------------------------------------------

# Set seed to make code reproducible => 2020
seed <- 2020
set.seed(seed)

# Parameters common to all estimations
# You should keep nb_sim=50 and nb_groups=5 to retrieve results
nb_sim = 50 # Number of simulations
nb_groups = 5 #Number of groups for GATES

# List of machine learning methods you want to test
# Possible : "glmnet","gbm","rf", "xgbTree", 'pcaNNet', 'svmPoly'
# For this project : 'rf', 'glmnet','gbm'
list_methods = c('rf', 'glmnet', 'gbm', 'glmnet_exp') #'rf', 'glmnet', 'gbm',

cat('Possible outcome variables are the following : \n\n' ,
    colnames(data)[startsWith(colnames(data), 'y_')])

# List of outcomes variables (paste "y" at the front for all outcomes)
potential_outcomes = c(
  'w123w_mean_remun',
 'lw123_mean_income_total',
  'w123m_mean_employed',
  'w123w_mean_employed',
  'lexppc_total',
  'lexppc_food'
)
outcomes = paste0("y_", potential_outcomes)

# Treatment arms 
cat('Possible treatment arms are the following : \n\n' ,
    unique(data$treatment))

treatment_arms_num = 1
control_arms_num = 0

# Parameters for parallel computation
nb_clusters = 10
# If you want to re-run all ML estimations set this parameter to TRUE, if there are already computed set this param to FALSE
run_ml = FALSE

#-------------------------------------------------------
# 2.B Run Generic Machine Learning
#-------------------------------------------------------

for (outcome in outcomes) {
  cat('... Computing results for :', outcome)
  # # Parralelisation is made on sim, we recommand to use parralelised computation
  # However you can always turn off the parallel option (it will be long)
  cl   <- makeCluster(nb_clusters, outfile = "")
  registerDoParallel(cl)
  #--------------------------------------------------
  gml(
    data_name = data_name,
    # name of the data file to load
    name_of_experiment = name_of_experiment,
    # name of the experiment
    y = outcome,
    nb_groups = nb_groups,
    treatment_arms_num = treatment_arms_num,
    control_arms_num = control_arms_num,
    sim = nb_sim,
    methods = list_methods,
    run_ml = run_ml
  )
  #--------------------------------------------------
}

#########################################################
# 3. RESULTS VISUALIZATION
#########################################################

# This function will create a word file in the result folder of the experiment that summarizes all results
render(
  input = "results_djibouti.Rmd",
  output_format = "word_document",
  output_dir = paste0("results/", name_of_experiment, sep = '')
)
